*SHORT-TERM WITH CONTRACT
* In this model the objective in the
* short-term planning problem is to determine over a given time
* horizon, typically weeks or months, the types of contracts for
* the purchase of raw materials and sales of products in order to
* maximize the profit, which can be calculated by the data on
* sales revenues, operating costs, marketing costs, inventory costs,
* and shortfall penalties. The goal is to decide:
* (a) which contract to make in the purchase of the raw materials and
*     selling the products in each time period
* (b) whether the capacity of each process should be expanded or not
*     in each time period.
* Reference: Minhwan Park, Sunwon Park, Fernando Mele and Ignacio E. Grossmann
*        "Modeling of Purchase and Sales Contracts in Supply Chain Optimization"
*        Ind. Eng. Chem. Res. 2006, 45, 5013-5026


* Additional constraints for considering demand lower bounds are added
* Additional constraints for inventory capacities are included
* Measure units are reviewed

option limrow =0, limcol=0,  optcr=0;
*solprint =off;
$ontext

      1---->O----2------> process 1 --9-->O----12-->
            |
            -----3------> ___________
                         | process 2 |--10->O--13-->
      4----------O--5--->|___________|      |
                 |8                         |
      6--O--7->process 3 ------------11---->

      streams of 1, 4, 6 are supplies and streams of 12, 13 demands
$offtext

sets    p          contracts length    /1*4/
        c          contract            /1*4/
        tp         more periods        /1*8/
        i          processes           /1*3/
        j          streams             /1*13/
        final(j)   final products      /12*13/
        rawmat(j)                      /1,4,6/
        t          periods             /1*6/;


parameters

excap(i) existing capacities: kton per year
/1 27
2 30
3 25/

cov(i)
/1 0.9, 2 0.85, 3 0.8/

 table spdm(j,t) kton per t of supply and demand limitations
     1    2    3    4    5    6
 1   100  100  100  100  100  100
 4   30   30   30   30   30   30
 6   100  100  100  100  100  100
 12  20   25   22   30   28   26
 13  51   50   53   60   59   50

table lbdm(j,t) demand lower bounds kton per t
     1   2   3   4   5   6
 12  5   5   5   5   5   5
 13  5   5   5   5   5   5

table delta(i,t) operating expense per unit of main product: 1e2 dollar per ton
  1    2    3    4    5    6
1 0.6  0.7  0.6  0.6  0.7  0.7
2 0.5  0.5  0.5  0.4  0.4  0.5
3 0.6  0.6  0.5  0.6  0.6  0.5

table pf(j,t) prices of the raw materials and products 1e2 dollar per ton
   1     2     3     4     5     6
1  2.2   2.4   2.4   2.3   2.2   2.2
4  1.9   2.4   2.4   2.2   2.1   2.1
6  5.2   5.7   5.5   5.4   5.7   5.7
12 22.1  23.9  24.4  22.7  27.9  23.6
13 20.5  21.5  24.5  21.2  22.8  24.9

table pd1(j,t) prices with discount contract. scheme 1. 1e2 dollar per ton
    1*6
 1  2.25
 4  2.30
 6  5.5

table pd2(j,t) prices with discount contract. scheme 2. 1e2 dollar per ton
    1*6
1   1
4   1
6   2

table pb1(j,t) prices with bulk contract. scheme 1. 1e2 dollar per ton
   1*6
1  2.1
4  2.35
6  5.5

table pb2(j,t) prices with bulk contract. scheme 2. 1e2 dollar per ton
     1*6
1   0.5
4   0.5
6   1

table pl(j,p,t) prices with length contract. 1e2 dollar per ton
    1.1*6    2.1*6     3.1*6
1   2.25
2             1.5
4   2.35      2.0        1.5
6   5.5       3          2

table lub(j,p) upper bound of quantities in the length contract in kton
    1     2     3
1  1000  1000  1000
4  1000  1000  1000
6  1000  1000  1000

*The next three tables don't have the same data that in the original document

table llb(j,p) lower bound of quantities in the length contract in kton
   1    2      3
1  0    62     66
4  0    3      4
6  0    11     24

table sigmad(j,t) lower bound of quantities in the discount contract in kton
     1*6
1    55
4    1
6    5

table sigmab(j,t) lower bound of quantities in the bulk contract in kton
     1*6
1    64
4    5
6    24


table pifk(i,j) i process needs j flows
     2   3   5   7
  1  1
  2      1   1
  3              1

  table jm(j,i) flow j is the main product of process i
      1   2   3
  9   1
  10      1
  11          1

  table psf(j,t) penalty of shortfall 1e2 dollar per ton
       1*6
  12   2
  13   2

  table sfub(j,t)  upper bound of shortfall per kton
         1*6
  12     10
  13     10

  table cinv(j,t) cost of inventory 1e2 dollar per ton
       1*6
  12   2
  13   2

  table invub(j,t) upper bound of inventory per kton
        1*6
  12     30
  13     30 ;

  variable prof profit in 1e2 dollar - objective variable
  positive variables
  f(j,t)           mass flow rates in tons per time interval t
  inv(j,t)         inventories in tons per time interval t
  sf(j,t)          shortfall in tons per time interval t

  spf(j,t)                       amount of fixed contract
  spd(j,t), spd1(j,t), spd2(j,t) amount of discount contract
  spb(j,t)                       amount of bulk contract
  spl(j,t)                       amount of variable lenght contract
  costpl(j,t)                    cost of variable lenght contract
  costpf(j,t)                    cost of fixed contract
  costpd(j,t)                    cost of discount contract
  costpb(j,t)                    cost of bulk contract
*  rev              revenues in 1e2 dollar
*  ope, pur, inve, sho;
;
  Binary variables
  yf(j,t)                      decision of fixed contract
  yd1(j,t),yd2(j,t), yd3(j,t)  decision of discount contract
  yb1(j,t),yb2(j,t), yb3(j,t)  decision of bulk contract
  yl(j,t) ,ylp(j,p,t)          decision of contract with variable length
   ;

  equations
* constraints independent of discrete choices
             obj
             N1(t),N2(t),N3(t),N4(t)                 node balance
             INVT11(t),INVT12(t),INVT21(t),INVT22(t) inventory balance
             invcap(j,t)                  maximum capacity of inventories
             shortfall(j,t)
             sfcap(j,t)
             bald11(j,t)                  balance for discount contract
             p1(t),p2(t),p3(t),p4(t)      process balance
             prcp1(t), prcp2(t), prcp3(t) production cannot exceed capacity
             d1(j,t),d2(j,t),d3(j,t)      products demand and supply limitations
             mb1(j,t)                     mass balance in period t between contract and flow rate
             sum_y(j,t)                   constraint to keep y var. in the model (see LogMIP manual)
* disjunction's constraints
             ubf(j,t),  ubfno(j,t)              fixed contract
             ubd1(j,t), ubd2(j,t), ubdno(j,t)   discount contract
             ubb1(j,t), ubb2(j,t), ubbno(j,t)   bulk contract
             ubd3(j,t), ubd4(j,t)               bulk contract
             lb1(j,t),  lb21(j,t), lb22(j,t)    lenght contract
             lb31(j,t), lb32(j,t), lb33(j,t)    lenght contract
             lengno1(j,t),    lengno2(j,t),     lengno3(j,t) lenght contract
             costplct1(j,t),  costplct21(j,t),  costplct22(j,t),
             costplct31(j,t), costplct32(j,t),  costplct33(j,t),
             costno1(j,t),    costno2(j,t),     costno3(j,t),
             costopfct(j,t),  costopdct(j,t),   costobct1(j,t),
             costobct2(j,t),  costobno(j,t) ;

  obj .. prof =e=
* sale of products
             sum((j,t) $ final(j), pf(j,t)*f(j,t))
* purchase of raw material with contracts
            - sum((j,t) $ rawmat(j), costpf(j,t))
            - sum((j,t) $ rawmat(j), costpd(j,t))
            - sum((j,t) $ rawmat(j), costpb(j,t))
            - sum((j,t) $ rawmat(j), costpl(j,t))
* production costs
            - sum((i,t), delta(i,t)*sum(j $ jm(j,i),f(j,t)))
* shortfall and inventory costs on streams 12 and 13
            - sum((j,t), sf(j,t)*psf(j,t))-sum((j,t), inv(j,t)*cinv(j,t));


  mb1(j,t)  $ rawmat(j)..   spf(j,t) + spd(j,t) + spb(j,t) + spl(j,t)  =e= f(j,t);
  bald11(j,t)$rawmat(j)..  spd(j,t) =e= spd1(j,t) + spd2(j,t);

* constraints for fixed contract
  ubf(j,t)$rawmat(j)..       spf(j,t) =l= 1000000.;
  ubfno(j,t)$rawmat(j)..     spf(j,t) =e= 0 ;
  costopfct(j,t)$rawmat(j).. costpf(j,t)=e=pf(j,t)*spf(j,t);

* constraints for discount contract
  ubd1(j,t)$rawmat(j)..      spd1(j,t) =l= 1000*sigmad(j,t);
  ubd2(j,t)$rawmat(j)..      spd1(j,t) =e= 1000*sigmad(j,t);
  ubd3(j,t)$rawmat(j)..      spd2(j,t) =e=0;
  ubd4(j,t)$rawmat(j)..      spd2(j,t) =g=0;
  ubdno(j,t)$rawmat(j)..     spd1(j,t) =e= 0;
  costopdct(j,t)$rawmat(j).. costpd(j,t)=e=pd1(j,t)*spd1(j,t)+pd2(j,t)*spd2(j,t);

* constraints for bulk contract
  ubb1(j,t)$rawmat(j)..      spb(j,t) =l= 1000*sigmab(j,t);
  costobct1(j,t)$rawmat(j).. costpb(j,t)=e=pb1(j,t)*spb(j,t);
  ubb2(j,t)$rawmat(j)..      spb(j,t) =g= 1000*sigmab(j,t);
  costobct2(j,t)$rawmat(j).. costpb(j,t)=e=pb2(j,t)*spb(j,t);
  ubbno(j,t)$rawmat(j)..     spb(j,t) =e= 0;
  costobno(j,t)$rawmat(j)..  costpb(j,t)=e=0;

* constraints for the length contract
  lb1(j,t)  $ rawmat(j)..    spl(j,t)=g=0;
  costplct1(j,t)$rawmat(j).. costpl(j,t)=e=pl(j,'1',t)*spl(j,t);
*
  lb21(j,t) $ rawmat(j)..    1000*llb(j,'2') =l= spl(j,t);
  lb22(j,t+1) $ (rawmat(j) and (ord(t)le 5)).. spl(j,t+1) =g= 1000*llb(j,'2');
  costplct21(j,t)$rawmat(j).. costpl(j,t)=e=pl(j,'2',t)*spl(j,t);
  costplct22(j,t+1)$(rawmat(j) and (ord(t)le 5)).. costpl(j,t+1)=e=pl(j,'2',t)*spl(j,t+1);
*
  lb31(j,t)$rawmat(j)..     1000*llb(j,'3') =l= spl(j,t);
  lb32(j,t+1)$(rawmat(j) and (ord(t) le 5)).. 1000*llb(j,'3') =l= spl(j,t+1);
  lb33(j,t+2)$(rawmat(j) and (ord(t) le 4)).. 1000*llb(j,'3') =l= spl(j,t+2);
  costplct31(j,t)$rawmat(j).. costpl(j,t)=e=pl(j,'3',t)*spl(j,t);
  costplct32(j,t+1)$(rawmat(j) and (ord(t)le 5)).. costpl(j,t+1)=e=pl(j,'3',t)*spl(j,t+1);
  costplct33(j,t+2)$(rawmat(j) and (ord(t)le 4)).. costpl(j,t+2)=e=pl(j,'3',t)*spl(j,t+2);
  lengno1(j,t)$rawmat(j)..  spl(j,t) =e=0;
  lengno2(j,t+1)$(rawmat(j) and (ord(t)le 5))..  spl(j,t+1) =e=0;
  lengno3(j,t+2)$(rawmat(j) and (ord(t)le 4))..  spl(j,t+2) =e=0;
  costno1(j,t)$rawmat(j)..  costpl(j,t)=e=0;
  costno2(j,t+1)$(rawmat(j) and (ord(t)le 5))..  costpl(j,t+1)=e=0;
  costno3(j,t+2)$(rawmat(j) and (ord(t)le 4))..  costpl(j,t+2)=e=0;

*
  sum_y(j,t)$rawmat(j)..  yf(j,t) + yd1(j,t) + yd2(j,t) +yd3(j,t)
                          + yb1(j,t) + yb2(j,t) + yb3(j,t) +
                          sum(p, ylp(j,p,t)) =g=0;

* mass balance equations for each node
  N1(T).. F('1',T) =E= F('2',T)+F('3',T);
  N2(T).. F('4',T) + F('8',T) =E= F('5',T);
  N3(T).. F('6',T) =E= F('7',T);
  N4(T).. F('3',T) =E= 10*F('5',T);

  P1(T)..  F('9',T) =E= cov('1')*F('2',T);
  P2(T)..  F('10',T) =E= cov('2')*(F('5',T)+f('3',t));
  P3(T)..  F('8',T) =E= 0.17*cov('3')*f('7',T);
  P4(T)..  F('11',t) =E= 0.83*cov('3')*F('7',T);

  prcp1(t).. f('9',t) =l= excap('1')*1000;
  prcp2(t).. f('10',t) =l= excap('2')*1000;
  prcp3(t).. f('11',t)+f('8',t) =l= excap('3')*1000;

  INVT11(t)$(ord(t) = 1)..  F('9',t) =E= F('12',t)+ inv('12',t);
  INVT12(t)$(ord(t) gt 1).. inv('12',t-1) + F('9',t) =E= F('12',t)+ inv('12',t);
  INVT21(t)$(ord(t) = 1)..  F('10',t) + F('11',t) =E= inv('13',t) + F('13',t);
  INVT22(t)$(ord(t) gt 1).. inv('13',t-1) + F('10',t) + F('11',t) =E= inv('13',t) + F('13',t);

* Maximum capacities of inventories
  invcap(j,t) $ final(j) .. inv(j,t)=l=invub(j,t)*1000;

* Shortfall calculation
 shortfall(j,t) $ final(j).. sf(j,t) =E= spdm(j,t)*1000 - f(j,t);

* Maximum shortfall allowed
  sfcap(j,t) $ final(j) .. sf(j,t)=l=sfub(j,t)*1000;

* Maximum capacities of suppliers
  D1(j,t) $ rawmat(j)..  f(j,T) =l= spdm(j,t)*1000;

* Upper bound for the demand
  D2(j,t) $ final(j)..  f(j,t) =l= spdm(j,t)*1000;
* Lower bound for the demand
  D3(j,t) $ final(j)..  f(j,t) =g= lbdm(j,t)*1000;

* Upper bounds for disjunction's variables -needed for  convex-hull (logmip)
  spf.up(j,t) =100000.;
  spd.up(j,t) =100000.;
  spd1.up(j,t)=100000.;
  spd2.up(j,t)=100000.;
  spb.up(j,t) =100000.;
  spl.up(j,t) =100000.;
  costpl.up(j,t)=10e6;
  costpf.up(j,t)=10e6;
  costpd.up(j,t)=10e6;
  costpb.up(j,t)=10e6;

$ONECHO > "%lm.info%"

      DISJUNCTION FXCT(J,T), DISC(J,T), BULK(J,T), LENG1(J), LENG2(J), LENG3(J),
                                       LENG4(J), LENG5(J) , LENG6(J) ;

      FXCT(J,T) with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YF(j,T) THEN
            ubf(j,t);
      ELSE
            ubfno(j,t);
      ENDIF ;



      DISC(j,t) with  (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YD1(j,t) THEN
            ubd1(j,t);
            ubd3(j,t);
      ELSIF YD2(j,t) THEN
            ubd2(j,t);
            ubd4(j,t);
      ELSIF YD3(j,t) THEN
            ubdno(j,t);
            ubd2(j,t);
      ENDIF ;



      BULK(j,t) with  (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YB1(j,t) THEN
            ubb1(j,t);
            costobct1(j,t);
      ELSIF YB2(j,t) THEN
            ubb2(j,t);
            costobct2(j,t);
      ELSIF YB3(j,t)  THEN
            ubbno(j,t);
            costobno(j,t);
      ENDIF ;

      LENG1(j)  with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YLP(j,'1','1') THEN
            lb1(j,'1');
            costplct1(j,'1');
      ELSIF YLP(j,'2','1')  THEN
            lb21(j,'1');
            lb22(j,'2');
            costplct21(j,'1') ;
            costplct22(j,'2') ;
      ELSIF  YLP(j,'3','1')  THEN
            lb31(j,'1') ;
            lb32(j,'2') ;
            lb33(j,'3') ;
            costplct31(j,'1');
            costplct32(j,'2');
            costplct33(j,'3');
      ELSIF YLP(j,'4','1')  THEN
               lengno1(j,'1');
               lengno2(j,'2');
               lengno3(j,'3');
               costno1(j,'1');
               costno2(j,'2');
               costno3(j,'3');
      ENDIF ;

      LENG2(j)  with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YLP(j,'1','2') THEN
            lb1(j,'2');
            costplct1(j,'2');
      ELSIF YLP(j,'2','2')  THEN
            lb21(j,'2');
            lb22(j,'3');
            costplct21(j,'2') ;
            costplct22(j,'3') ;
      ELSIF  YLP(j,'3','2')  THEN
            lb31(j,'2') ;
            lb32(j,'3') ;
            lb33(j,'4') ;
            costplct31(j,'2');
            costplct32(j,'3');
            costplct33(j,'4');
      ELSIF YLP(j,'4','2')  THEN
               lengno1(j,'2');
               lengno2(j,'3');
               lengno3(j,'4');
               costno1(j,'2');
               costno2(j,'3');
               costno3(j,'4');
      ENDIF ;

      LENG3(j)  with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YLP(j,'1','3') THEN
            lb1(j,'3');
            costplct1(j,'3');
      ELSIF YLP(j,'2','3')  THEN
            lb21(j,'3');
            lb22(j,'4');
            costplct21(j,'3') ;
            costplct22(j,'4') ;
      ELSIF  YLP(j,'3','3')  THEN
            lb31(j,'3') ;
            lb32(j,'4') ;
            lb33(j,'5') ;
            costplct31(j,'3');
            costplct32(j,'4');
            costplct33(j,'5');
      ELSIF YLP(j,'4','3')  THEN
               lengno1(j,'3');
               lengno2(j,'4');
               lengno3(j,'5');
               costno1(j,'3');
               costno2(j,'4');
               costno3(j,'5');
      ENDIF ;

      LENG4(j)  with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YLP(j,'1','4') THEN
            lb1(j,'4');
            costplct1(j,'4');
      ELSIF YLP(j,'2','4')  THEN
            lb21(j,'4');
            lb22(j,'5');
            costplct21(j,'4') ;
            costplct22(j,'5') ;
      ELSIF  YLP(j,'3','4')  THEN
            lb31(j,'4') ;
            lb32(j,'5') ;
            lb33(j,'6') ;
            costplct31(j,'4');
            costplct32(j,'5');
            costplct33(j,'6');
      ELSIF YLP(j,'4','4')  THEN
               lengno1(j,'4');
               lengno2(j,'5');
               lengno3(j,'6');
               costno1(j,'4');
               costno2(j,'5');
               costno3(j,'6');
      ENDIF ;

      LENG5(j)  with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YLP(j,'1','5') THEN
            lb1(j,'5');
            costplct1(j,'5');
      ELSIF YLP(j,'2','5')  THEN
            lb21(j,'5');
            lb22(j,'6');
            costplct21(j,'5') ;
            costplct22(j,'6') ;
      ELSIF  YLP(j,'3','5')  THEN
            lb31(j,'5') ;
            lb32(j,'6') ;
            costplct31(j,'5');
            costplct32(j,'6');
      ELSIF YLP(j,'4','5')  THEN
               lengno1(j,'5');
               lengno2(j,'6');
               costno1(j,'5');
               costno2(j,'6');
      ENDIF ;


      LENG6(j)  with (ord(j) eq 1) or (ord(j) eq 4) or (ord(j) eq 6) is
      IF YLP(j,'1','6') THEN
            lb1(j,'6');
            costplct1(j,'6');
      ELSIF YLP(j,'2','6')  THEN
            lb21(j,'6');
            costplct21(j,'6') ;
      ELSIF  YLP(j,'3','6')  THEN
            lb31(j,'6') ;
            costplct31(j,'6');
      ELSIF YLP(j,'4','6')  THEN
               lengno1(j,'6');
               costno1(j,'6');
      ENDIF ;

$offecho
  model mulplan /all/;
  option mip = lmchull;
  option iterlim = 10000000;
  mulplan.nodlim = 10000000;
  mulplan.optfile=1;
  solve mulplan using mip maximizing prof;
  display inv.l, sf.l, prof.l,  f.l,
          yf.l, yd1.l, yd2.l,yd3.l,
          yb1.l, yb2.l, yb3.l,
          ylp.l,
          spf.l, spd.l,
          spb.l, spl.l,
          costpf.l, costpd.l, costpb.l,costpl.l;