$TITLE                                        RETROFIT_BIGM (10 PROCESS)
$OFFSYMXREF
$OFFSYMLIST
$ONINLINE

* Retrofit problem solved by Nick Sawaya
* Reference:  Jackson J. and Grossmann I.E.,
*        "A cutting plane method for solving linear generalized
*        disjunctive programming problems",
*         Comp.&Chem. Eng. Volume 29, Issue 9, 2005, Pages 1891-1913
*
* See process flowsheet for stream labeling *
******************************************************* SETS
***********************************************************

SETS

S            /1*37/                       /*Streams*/
P            /1*10/                       /*Processes*/
N            /1*15/                       /*Nodes (Not used in code below. Listed
here only for completeness)*/
T            /1*4/                        /*Time periods*/
M            /1*4/                        /*Conversion &OR capacity scenarios*/
MOD          /1*2/                        /*Energy reduction scenarios*/
K            /1*16/                       /*Temperature intervals (Derived once
cold and hot streams have been identified)*/

S_PROD(S)   /5,9,20,21,26,28,37/          /*Product streams*/
S_RAW(S)    /1,6,10,22,29/                /*Raw feed streams*/

S_P1_IN(S)  /2/                           /*Streams into process Pi*/
S_P1_OUT(S) /4/                                  /*Reacted streams out of process Pi*/
S_P2_IN(S)  /3,7/
S_P2_OUT(S) /8/
S_P3_IN(S)  /11/
S_P3_OUT(S) /12,13/
S_P4_IN(S)  /14/
S_P4_OUT(S) /15/
S_P5_IN(S)  /17/
S_P5_OUT(S) /18,19/
S_P6_IN(S)  /16,23/
S_P6_OUT(S) /25/
S_P7_IN(S)  /24,32/
S_P7_OUT(S) /27/
S_P8_IN(S)  /30/
S_P8_OUT(S) /31/
S_P9_IN(S)  /33/
S_P9_OUT(S) /34/
S_P10_IN(S)  /35/
S_P10_OUT(S) /36/

S_N1_IN(S)  /1/                           /*Streams into node Ni*/
S_N1_OUT(S) /2,3/                         /*Streams out of node Ni*/
S_N2_IN(S)  /4/
S_N2_OUT(S) /5/
S_N3_IN(S)  /6,12/
S_N3_OUT(S) /7/
S_N4_IN(S)  /8,13/
S_N4_OUT(S) /9/
S_N5_IN(S)  /10/
S_N5_OUT(S) /11,14/
S_N6_IN(S)  /15/
S_N6_OUT(S) /16,17/
S_N7_IN(S)  /18/
S_N7_OUT(S) /20/
S_N8_IN(S)  /19/
S_N8_OUT(S) /21/
S_N9_IN(S)  /22/
S_N9_OUT(S) /23,24/
S_N10_IN(S)  /25/
S_N10_OUT(S) /26/
S_N11_IN(S)  /27/
S_N11_OUT(S) /28/
S_N12_IN(S)  /29/
S_N12_OUT(S) /30/
S_N13_IN(S)  /31/
S_N13_OUT(S) /32,33/
S_N14_IN(S)  /34/
S_N14_OUT(S) /35/
S_N15_IN(S)  /36/
S_N15_OUT(S) /37/

S_COLD(S_RAW) /1,6,10,22,29/             /*Cold streams (Chosen by user)*/
S_HOT(S_PROD) /5,9,20,21,26,28,37/       /*Hot streams (Chosen by user)*/
;

ALIAS

(T,TAU)
;

***************************************************** VARIABLES
*******************************************************

VARIABLES

obj                                      /*Objective function variable (in$)*/

mf(S,T)                                  /*Mass flowrate (in 1E6 lbs per time period)*/
f(S,T)                                   /*Molar flowrate (in 1E3 lbmol per time period)*/
mf_p1_unreacted(T)                       /*Mass flowrate of unreacted streams for  process Pi (in 1E6 lbs per time period)*/
mf_p2_unreacted(T)
mf_p3_unreacted(T)
mf_p4_unreacted(T)
mf_p5_unreacted(T)
mf_p6_unreacted(T)
mf_p7_unreacted(T)
mf_p8_unreacted(T)
mf_p9_unreacted(T)
mf_p10_unreacted(T)

q_cold(S_COLD,K,T)     /*Heat content of cold streams (in 1E6 BTU)*/
q_hot(S_HOT,K,T)       /*Heat content of hot streams (in  1E6 BTU)*/
q_steam(T)             /*Heat content of total steam required (in 1E6 BTU)*/
q_water(T)             /*Heat content of total water required (in 1E6 BTU)*/
q_steam_int(K,T)       /*Heat content of steam required perinterval (in 1E6 BTU)*/
q_water_int(K,T)       /*Heat content of water required perinterval (in 1E6 BTU)*/
r(K,T)                 /*Heat content of residuals per interval(in 1E6 BTU)*/

convcapcost(P,T)       /*Retrofit conversion &OR capacity fixed cost (in 1E3 $)*/
energycost(T)          /*Retrofit energy reduction fixed cost (in 1E3 $)*/

y(P,M,T)               /*Retrofit variable for conversion &OR capacity*/
w(P,M,T)               /*Retrofit variable for conversion &OR capacity fixed cost enforcement*/
x(MOD,T)               /*Retrofit variable for energy reduction*/
v(MOD,T)               /*Retrofit variable for energy reduction fixed cost enforcement*/
;

POSITIVE VARIABLES

mf(S,T)
f(S,T)
mf_p1_unreacted(T)
mf_p2_unreacted(T)
mf_p3_unreacted(T)
mf_p4_unreacted(T)
mf_p5_unreacted(T)
mf_p6_unreacted(T)
mf_p7_unreacted(T)
mf_p8_unreacted(T)
mf_p9_unreacted(T)
mf_p10_unreacted(T)

q_cold(S_COLD,K,T)
q_hot(S_HOT,K,T)
q_steam(T)
q_water(T)
q_steam_int(K,T)
q_water_int(K,T)
r(K,T)

convcapcost(P,T)
energycost(T)
;

BINARY VARIABLES

y(P,M,T)
w(P,M,T)
x(MOD,T)
v(MOD,T)
;

****************************************************** DATA ENTRY
*****************************************************

******************************************* Prices, Costs and Dollar Limits

SCALAR Price_Steam /10/;                  /*Price of steam (in $ per 1E6
BTU)*/
SCALAR Price_Water /2.5/;                 /*Price of water (in $ per 1E6
BTU)*/

TABLE Price_Prod(S_PROD,T)                /*Price of Product Streams (in 1E3
$ per 1E6 lbs)*/
      1      2      3      4
5     120    110    110    100
9     200    190    170    160
20    250    240    240    240
21    300    260    250    250
26    350    340    330    330
28    280    270    250    250
37    400    350    320    320
;

TABLE Price_Raw(S_RAW,T)                  /*Price of Raw Streams (in 1E3 of $ per 1E6
lbs)*/
      1      2      3      4
1     10     7      5      5
6     15     11     9      9
10    18     14     10     9
22    19     17     17     16
29    16     16     15     12
;

TABLE FixedCost_ConvCap(P,M,T)            /*Fixed Cost for Retrofit
Conversion &OR Capacity (in 1E3 $)*/
        1        2        3        4
1.1     0        0        0        0
2.1     0        0        0        0
3.1     0        0        0        0
4.1     0        0        0        0
5.1     0        0        0        0
6.1     0        0        0        0
7.1     0        0        0        0
8.1     0        0        0        0
9.1     0        0        0        0
10.1    0        0        0        0
1.2     60       40       30       30
2.2     70       40       40       40
3.2     70       50       30       30
4.2     110      80       60       50
5.2     100      70       60       60
6.2     90       90       70       60
7.2     80       70       70       40
8.2     80       60       50       30
9.2     90       80       80       70
10.2    70       70       60       60
1.3     400      350      200      200
2.3     300      250      200      200
3.3     150      50       25       20
4.3     130      80       30       30
5.3     130      80       30       25
6.3     300      300      250      200
7.3     200      150      100      100
8.3     150      100      60       30
9.3     200      180      150      100
10.3    250      230      200      140
1.4     460      390      230      230
2.4     370      290      220      240
3.4     220      100      55       50
4.4     240      160      90       80
5.4     230      150      90       85
6.4     390      390      320      260
7.4     280      220      170      140
8.4     230      160      110      60
9.4     290      260      230      170
10.4    320      300      260      200
;

TABLE FixedCost_Energy(MOD,T)             /*Fixed cost for (no) energy
reduction (in 1E3 $)*/
        1     2     3     4
1       0     0     0     0
2       20    15    15    15
;

PARAMETER InvestmentLimit(T)              /*Limit on Total Investment (in
1E3 $ per period)*/
/ 1 = 4000
  2 = 3800
  3 = 3600
  4 = 3500/
;

******************************************* Physical Properties
*Molecular Weight of Stream (in 1E6 lb per 1E3 lbmol)
PARAMETER MolWeight(S)
/ 1 = 0.2
  2 = 0.2
  3 = 0.2
  4 = 0.2
  5 = 0.2
  6 = 0.5
  7 = 0.5
  8 = 0.7
  9 = 0.7
  10 = 1.2
  11 = 1.2
  12 = 0.5
  13 = 0.7
  14 = 1.2
  15 = 1.2
  16 = 1.2
  17 = 1.2
  18 = 0.3
  19 = 0.9
  20 = 0.3
  21 = 0.9
  22 = 0.4
  23 = 0.4
  24 = 0.4
  25 = 1.6
  26 = 1.6
  27 = 1.1
  28 = 1.1
  29 = 0.7
  30 = 0.7
  31 = 0.7
  32 = 0.7
  33 = 0.7
  34 = 0.7
  35 = 0.7
  36 = 0.7
  37 = 0.7/
;

PARAMETER Gamma(S)                               /*Stoichiometric Coefficients.
Note: In this instance of the problem, we assume all relevant stoic
coefficients to be 1*/

/ 1 = 0
  2 = 1
  3 = 1
  4 = 1
  5 = 0
  6 = 0
  7 = 1
  8 = 1
  9 = 0
 10 = 0
 11 = 1
 12 = 1
 13 = 1
 14 = 1
 15 = 1
 16 = 1
 17 = 1
 18 = 1
 19 = 1
 20 = 0
 21 = 0
 22 = 0
 23 = 1
 24 = 1
 25 = 1
 26 = 0
 27 = 1
 28 = 0
 29 = 0
 30 = 1
 31 = 1
 32 = 1
 33 = 1
 34 = 1
 35 = 1
 36 = 1
 37 = 0/
;

PARAMETER Cp_Cold(S_COLD)                 /*Heat Capacity for Cold Streams
(in 1E6 BTU per 1E6 lb .F)*/
/ 1 = 0.1
  6 = 0.6
 10 = 1
 22 = 0.8
 29 = 0.7/
;

PARAMETER Cp_Hot(S_HOT)                   /*Heat Capacity for Hot Streams
(in 1E6 BTU per 1E6 lb .F)*/
/ 5 = 0.8
  9 = 0.7
 20 = 0.9
 21 = 0.45
 26 = 0.75
 28 = 0.65
 37 = 0.5/
;

TABLE Temp_Str_Cold_In(S_COLD,T)          /*Inlet Temperature of Cold Stream
(in F)*/
        1    2     3     4
1      100   100   100   100
6      120   120   120   120
10     150   150   150   150
22     110   110   110   110
29     140   140   140   140
;

TABLE Temp_Str_Cold_Out(S_COLD,T)         /*Outlet Temperature of Cold
Stream (in F)*/
        1    2     3     4
1      630   630   630   630
6      530   530   530   530
10     520   520   520   520
22     600   600   600   600
29     550   550   550   550
;

TABLE Temp_Int_Cold_In(K,T)               /*Inlet Temperature of Cold
Interval (in F)*/
        1    2     3     4
1      630   630   630   630
2      600   600   600   600
3      590   590   590   590
4      560   560   560   560
5      550   550   550   550
6      540   540   540   540
7      530   530   530   530
8      520   520   520   520
9      190   190   190   190
10     180   180   180   180
11     170   170   170   170
12     160   160   160   160
13     150   150   150   150
14     140   140   140   140
15     120   120   120   120
16     110   110   110   110
;

TABLE Temp_Int_Cold_Out(K,T)              /*Outlet Temperature of Cold
Interval (in F)*/
        1    2     3     4
1      600   600   600   600
2      590   590   590   590
3      560   560   560   560
4      550   550   550   550
5      540   540   540   540
6      530   530   530   530
7      520   520   520   520
8      190   190   190   190
9      180   180   180   180
10     170   170   170   170
11     160   160   160   160
12     150   150   150   150
13     140   140   140   140
14     120   120   120   120
15     110   110   110   110
16     100   100   100   100
;

TABLE Temp_Str_Hot_In(S_HOT,T)            /*Inlet Temperature of Hot Stream
(in F)*/
        1    2     3     4
5      610   610   610   610
9      600   600   600   600
20     550   550   550   550
21     550   550   550   550
26     570   570   570   570
28     560   560   560   560
37     600   600   600   600
;

TABLE Temp_Str_Hot_Out(S_HOT,T)           /*Outlet Temperature of Hot Stream
(in F)*/
        1    2     3     4
5      200   200   200   200
9      190   190   190   190
20     170   170   170   170
21     170   170   170   170
26     180   180   180   180
28     150   150   150   150
37     150   150   150   150
;

TABLE Temp_Int_Hot_In(K,T)                /*Inlet Temperature of Hot
Interval (in F)*/
       1     2     3     4
1      640   640   640   640
2      610   610   610   610
3      600   600   600   600
4      570   570   570   570
5      560   560   560   560
6      550   550   550   550
7      540   540   540   540
8      530   530   530   530
9      200   200   200   200
10     190   190   190   190
11     180   180   180   180
12     170   170   170   170
13     160   160   160   160
14     150   150   150   150
15     130   130   130   130
16     120   120   120   120
;

TABLE Temp_Int_Hot_Out(K,T)               /*Outlet Temperature of Hot
Interval (in F)*/
       1     2     3     4
1      610   610   610   610
2      600   600   600   600
3      570   570   570   570
4      560   560   560   560
5      550   550   550   550
6      540   540   540   540
7      530   530   530   530
8      200   200   200   200
9      190   190   190   190
10     180   180   180   180
11     170   170   170   170
12     160   160   160   160
13     150   150   150   150
14     130   130   130   130
15     120   120   120   120
16     110   110   110   110
;

PARAMETER DeltaT_Cold(S_COLD,K,T)         /*Temperature difference for every
cold stream in every cold interval (in F)*/
/ 1  .1*16 .1*4 = 0
  6  .1*16 .1*4 = 0
  10 .1*16 .1*4 = 0
  22 .1*16 .1*4 = 0
  29 .1*16 .1*4 = 0/
;

LOOP((S_COLD,K,T),
        DeltaT_Cold(S_COLD,K,T)$((Temp_Str_Cold_Out(S_COLD,T) ge
Temp_Int_Cold_In(K,T)) AND (Temp_Str_Cold_In(S_COLD,T) le
Temp_Int_Cold_Out(K,T))) = Temp_Int_Cold_In(K,T) - Temp_Int_Cold_Out(K,T);
);

PARAMETER DeltaT_Hot(S_HOT,K,T)           /*Temperature difference for every
hot stream in every hot interval (in F)*/
/ 5  .1*16 .1*4 = 0
  9  .1*16 .1*4 = 0
  20 .1*16 .1*4 = 0
  21 .1*16 .1*4 = 0
  26 .1*16 .1*4 = 0
  28 .1*16 .1*4 = 0
  37 .1*16 .1*4 = 0/
;

LOOP((S_HOT,K,T),
        DeltaT_Hot(S_HOT,K,T)$((Temp_Str_Hot_In(S_HOT,T) ge Temp_Int_Hot_In(K,T))
AND (Temp_Str_Hot_Out(S_HOT,T) le Temp_Int_Hot_Out(K,T))) =
Temp_Int_Hot_In(K,T) - Temp_Int_Hot_Out(K,T);
);

******************************************* Design Specs

TABLE Dem(S_PROD,T)                       /*Demand for Product (in 1E6 lbs per time
period)*/
        1       2       3        4
5       0.2     0.15    0.1      0.1
9       2       1.5     1        0.9
20      0.1     0.1     0.1      0.1
21      0.1     0.1     0.1      0.1
26      0.4     0.3     0.2      0.15
28      0.3     0.2     0.1      0.1
37      0.9     0.8     0.6      0.5
;

TABLE Sup(S_RAW,T)                        /*Supply of raw feed (in 1E6 lbs per time
period)*/
        1       2       3        4
1       3.5     3       3        3
6       3.6     3.1     3        2.8
10      2.5     2.2     2.2      2
22      2.4     2.1     2        2
29      3       2.5     2.1      1.9
;

TABLE Eta(P,M,T)                          /*Conversion Rate*/
        1       2       3        4
1.1     0.8     0.8     0.8      0.8
2.1     0.9     0.9     0.9      0.9
3.1     0.85    0.85    0.85     0.85
4.1     0.85    0.85    0.85     0.85
5.1     0.75    0.75    0.75     0.75
6.1     0.8     0.8     0.8      0.8
7.1     0.85    0.85    0.85     0.85
8.1     0.8     0.8     0.8      0.8
9.1     0.78    0.78    0.78     0.78
10.1    0.8     0.8     0.8      0.8
1.2     0.85    0.85    0.85     0.85
2.2     0.95    0.95    0.95     0.95
3.2     0.98    0.98    0.98     0.98
4.2     0.9     0.9     0.9      0.9
5.2     0.95    0.95    0.95     0.95
6.2     0.85    0.85    0.85     0.85
7.2     0.95    0.95    0.95     0.95
8.2     0.92    0.92    0.92     0.92
9.2     0.85    0.85    0.85     0.85
10.2    0.95    0.95    0.95     0.95
1.3     0.8     0.8     0.8      0.8
2.3     0.9     0.9     0.9      0.9
3.3     0.85    0.85    0.85     0.85
4.3     0.85    0.85    0.85     0.85
5.3     0.9     0.9     0.9      0.9
6.3     0.8     0.8     0.8      0.8
7.3     0.85    0.85    0.85     0.85
8.3     0.8     0.8     0.8      0.8
9.3     0.78    0.78    0.78     0.78
10.3    0.8     0.8     0.8      0.8
1.4     0.85    0.85    0.85     0.85
2.4     0.95    0.95    0.95     0.95
3.4     0.98    0.98    0.98     0.98
4.4     0.9     0.9     0.9      0.9
5.4     0.95    0.95    0.95     0.95
6.4     0.85    0.85    0.85     0.85
7.4     0.95    0.95    0.95     0.95
8.4     0.92    0.92    0.92     0.92
9.4     0.85    0.85    0.85     0.85
10.4    0.95    0.95    0.95     0.95
;

TABLE Cap(P,M,T)                         /*Capacity limit on every process (in 1E6 lbs
per time period)*/
        1       2       3        4
1.1     1       1       1        1
2.1     4       4       4        4
3.1     1.5     1.5     1.5      1.5
4.1     1.5     1.5     1.5      1.5
5.1     1       1       1        1
6.1     2       2       2        2
7.1     2.5     2.5     2.5      2.5
8.1     1.5     1.5     1.5      1.5
9.1     2       2       2        2
10.1    2       2       2        2
1.2     1       1       1        1
2.2     4       4       4        4
3.2     1.5     1.5     1.5      1.5
4.2     1.5     1.5     1.5      1.5
5.2     1       1       1        1
6.2     2       2       2        2
7.2     2.5     2.5     2.5      2.5
8.2     1.5     1.5     1.5      1.5
9.2     2       2       2        2
10.2    2       2       2        2
1.3     5       5       5        5
2.3     6       6       6        6
3.3     2.5     2.5     2.5      2.5
4.3     2       2       2        2
5.3     2       2       2        2
6.3     5.5     5.5     5.5      5.5
7.3     5       5       5        5
8.3     3.5     3.5     3.5      3.5
9.3     4       4       4        4
10.3    5       5       5        5
1.4     5       5       5        5
2.4     6       6       6        6
3.4     2.5     2.5     2.5      2.5
4.4     2       2       2        2
5.4     2       2       2        2
6.4     5.5     5.5     5.5      5.5
7.4     5       5       5        5
8.4     3.5     3.5     3.5      3.5
9.4     4       4       4        4
10.4    5       5       5        5
;

*$GDXOUT data.gdx
*$UNLOAD

EQUATIONS

Objective

Equivalence_Mass(S,T)

Demand(S_PROD,T)
Supply(S_RAW,T)

MassBalance_Node1(T)
MassBalance_Node2(T)
MassBalance_Node3(T)
MassBalance_Node4(T)
MassBalance_Node5(T)
MassBalance_Node6(T)
MassBalance_Node7(T)
MassBalance_Node8(T)
MassBalance_Node9(T)
MassBalance_Node10(T)
MassBalance_Node11(T)
MassBalance_Node12(T)
MassBalance_Node13(T)
MassBalance_Node14(T)
MassBalance_Node15(T)

MassBalance_Process1(T)
MassBalance_Process2(T)
MassBalance_Process3(T)
MassBalance_Process4(T)
MassBalance_Process5(T)
MassBalance_Process6(T)
MassBalance_Process7(T)
MassBalance_Process8(T)
MassBalance_Process9(T)
MassBalance_Process10(T)

Limiting_Process2(T)
Limiting_Process6(T)
Limiting_Process7(T)

Disj1_Conv_Process1_lt(S_P1_OUT,M,T)
Disj1_Conv_Process1_gt(S_P1_OUT,M,T)
Disj1_Conv_Process2_lt(S_P2_OUT,M,T)
Disj1_Conv_Process2_gt(S_P2_OUT,M,T)
Disj1_Conv_Process3_lt(S_P3_OUT,M,T)
Disj1_Conv_Process3_gt(S_P3_OUT,M,T)
Disj1_Conv_Process4_lt(S_P4_OUT,M,T)
Disj1_Conv_Process4_gt(S_P4_OUT,M,T)
Disj1_Conv_Process5_lt(S_P5_OUT,M,T)
Disj1_Conv_Process5_gt(S_P5_OUT,M,T)
Disj1_Conv_Process6_lt(S_P6_OUT,M,T)
Disj1_Conv_Process6_gt(S_P6_OUT,M,T)
Disj1_Conv_Process7_lt(S_P7_OUT,M,T)
Disj1_Conv_Process7_gt(S_P7_OUT,M,T)
Disj1_Conv_Process8_lt(S_P8_OUT,M,T)
Disj1_Conv_Process8_gt(S_P8_OUT,M,T)
Disj1_Conv_Process9_lt(S_P9_OUT,M,T)
Disj1_Conv_Process9_gt(S_P9_OUT,M,T)
Disj1_Conv_Process10_lt(S_P10_OUT,M,T)
Disj1_Conv_Process10_gt(S_P10_OUT,M,T)

Disj1_Cap_Process1(M,T)
Disj1_Cap_Process2(M,T)
Disj1_Cap_Process3(M,T)
Disj1_Cap_Process4(M,T)
Disj1_Cap_Process5(M,T)
Disj1_Cap_Process6(M,T)
Disj1_Cap_Process7(M,T)
Disj1_Cap_Process8(M,T)
Disj1_Cap_Process9(M,T)
Disj1_Cap_Process10(M,T)

Disj2_lt(P,M,T)
Disj2_gt(P,M,T)

Equivalence_Energy_Cold(S_COLD,K,T)
Equivalence_Energy_Hot(S_HOT,K,T)

Disj3_NoEnergy_Steam_lt(T)
Disj3_NoEnergy_Steam_gt(T)
Disj3_NoEnergy_Water_lt(T)
Disj3_NoEnergy_Water_gt(T)

Disj3_Energy_Steam_lt(T)
Disj3_Energy_Steam_gt(T)
Disj3_Energy_Water_lt(T)
Disj3_Energy_Water_gt(T)
Disj3_Transshipment_lt(K,T)
Disj3_Transshipment_gt(K,T)

Disj4_NoEnergy_lt(T)
Disj4_NoEnergy_gt(T)

Disj4_Energy_lt(T)
Disj4_Energy_gt(T)

LimitingCost(T)

Sum_y(P,T)
Sum_w(P,T)
Sum_x(T)
Sum_v(T)

Logic_y(P,M,T,TAU)
Logic_w(P,M,T,TAU)
Logic_yw_M1(P,M,T)
Logic_yw_MlessM1(P,M,T)

Logic_x(T,TAU)
Logic_v(T,TAU)
Logic_xv_MOD1(T)
Logic_xv_MOD2(T)
;

****************************************************************************
********************************************
*************************************************** OPTIMIZATION PROBLEM
***********************************************
****************************************************************************
********************************************

*Note: We multiply, where appropriate, the coefficents in the objective
* function by 1E3 in order to obtain our obj fn value in $.

Objective.. obj =E=
1E3*(SUM(T,SUM(S_PROD,Price_Prod(S_PROD,T)*mf(S_PROD,T))) -
SUM(T,SUM(S_RAW,Price_Raw(S_RAW,T)*mf(S_RAW,T))))
                  - SUM(T,Price_Steam*(q_steam(T)) +
Price_Water*(q_water(T)))
                  -
1E3*(SUM(T,SUM(M,SUM(P,FixedCost_ConvCap(P,M,T)*w(P,M,T)))))
                  - 1E3*(SUM(T,SUM(MOD,FixedCost_Energy(MOD,T)*v(MOD,T))));

Equivalence_Mass(S,T).. mf(S,T) =E= f(S,T)*MolWeight(S);

Demand(S_PROD,T).. mf(S_PROD,T) =G= Dem(S_PROD,T);
Supply(S_RAW,T)..  mf(S_RAW,T) =L= Sup(S_RAW,T);

MassBalance_Node1(T)..  SUM(S_N1_IN,mf(S_N1_IN,T)) =E=
SUM(S_N1_OUT,mf(S_N1_OUT,T));
MassBalance_Node2(T)..  SUM(S_N2_IN,mf(S_N2_IN,T)) =E=
SUM(S_N2_OUT,mf(S_N2_OUT,T));
MassBalance_Node3(T)..  SUM(S_N3_IN,mf(S_N3_IN,T)) =E=
SUM(S_N3_OUT,mf(S_N3_OUT,T));
MassBalance_Node4(T)..  SUM(S_N4_IN,mf(S_N4_IN,T)) =E=
SUM(S_N4_OUT,mf(S_N4_OUT,T));
MassBalance_Node5(T)..  SUM(S_N5_IN,mf(S_N5_IN,T)) =E=
SUM(S_N5_OUT,mf(S_N5_OUT,T));
MassBalance_Node6(T)..  SUM(S_N6_IN,mf(S_N6_IN,T)) =E=
SUM(S_N6_OUT,mf(S_N6_OUT,T));
MassBalance_Node7(T)..  SUM(S_N7_IN,mf(S_N7_IN,T)) =E=
SUM(S_N7_OUT,mf(S_N7_OUT,T));
MassBalance_Node8(T)..  SUM(S_N8_IN,mf(S_N8_IN,T)) =E=
SUM(S_N8_OUT,mf(S_N8_OUT,T));
MassBalance_Node9(T)..  SUM(S_N9_IN,mf(S_N9_IN,T)) =E=
SUM(S_N9_OUT,mf(S_N9_OUT,T));
MassBalance_Node10(T).. SUM(S_N10_IN,mf(S_N10_IN,T)) =E=
SUM(S_N10_OUT,mf(S_N10_OUT,T));
MassBalance_Node11(T).. SUM(S_N11_IN,mf(S_N11_IN,T)) =E=
SUM(S_N11_OUT,mf(S_N11_OUT,T));
MassBalance_Node12(T).. SUM(S_N12_IN,mf(S_N12_IN,T)) =E=
SUM(S_N12_OUT,mf(S_N12_OUT,T));
MassBalance_Node13(T).. SUM(S_N13_IN,mf(S_N13_IN,T)) =E=
SUM(S_N13_OUT,mf(S_N13_OUT,T));
MassBalance_Node14(T).. SUM(S_N14_IN,mf(S_N14_IN,T)) =E=
SUM(S_N14_OUT,mf(S_N14_OUT,T));
MassBalance_Node15(T).. SUM(S_N15_IN,mf(S_N15_IN,T)) =E=
SUM(S_N15_OUT,mf(S_N15_OUT,T));

MassBalance_Process1(T).. SUM(S_P1_IN,mf(S_P1_IN,T)) =E=
SUM(S_P1_OUT,mf(S_P1_OUT,T)) + mf_p1_unreacted(T);
MassBalance_Process2(T).. SUM(S_P2_IN,mf(S_P2_IN,T)) =E=
SUM(S_P2_OUT,mf(S_P2_OUT,T)) + mf_p2_unreacted(T);
MassBalance_Process3(T).. SUM(S_P3_IN,mf(S_P3_IN,T)) =E=
SUM(S_P3_OUT,mf(S_P3_OUT,T)) + mf_p3_unreacted(T);
MassBalance_Process4(T).. SUM(S_P4_IN,mf(S_P4_IN,T)) =E=
SUM(S_P4_OUT,mf(S_P4_OUT,T)) + mf_p4_unreacted(T);
MassBalance_Process5(T).. SUM(S_P5_IN,mf(S_P5_IN,T)) =E=
SUM(S_P5_OUT,mf(S_P5_OUT,T)) + mf_p5_unreacted(T);
MassBalance_Process6(T).. SUM(S_P6_IN,mf(S_P6_IN,T)) =E=
SUM(S_P6_OUT,mf(S_P6_OUT,T)) + mf_p6_unreacted(T);
MassBalance_Process7(T).. SUM(S_P7_IN,mf(S_P7_IN,T)) =E=
SUM(S_P7_OUT,mf(S_P7_OUT,T)) + mf_p7_unreacted(T);
MassBalance_Process8(T).. SUM(S_P8_IN,mf(S_P8_IN,T)) =E=
SUM(S_P8_OUT,mf(S_P8_OUT,T)) + mf_p8_unreacted(T);
MassBalance_Process9(T).. SUM(S_P9_IN,mf(S_P9_IN,T)) =E=
SUM(S_P9_OUT,mf(S_P9_OUT,T)) + mf_p9_unreacted(T);
MassBalance_Process10(T).. SUM(S_P10_IN,mf(S_P10_IN,T)) =E=
SUM(S_P10_OUT,mf(S_P10_OUT,T)) + mf_p10_unreacted(T);

* Note: We add an additional constraint to ensure that conversion in every
* process is being done relative to limiting reactant (We assume limiting
* reactant is known apriori for every process).

Limiting_Process2(T).. f('3',T)*Gamma('3') =L= f('7',T)*Gamma('7');
Limiting_Process6(T).. f('16',T)*Gamma('16') =L= f('23',T)*Gamma('23');
Limiting_Process7(T).. f('24',T)*Gamma('24') =L= f('32',T)*Gamma('32');

*****************
*Disjunction 1 constraints

Disj1_Conv_Process1_lt(S_P1_OUT,M,T).. f(S_P1_OUT,T) =L=
                         f('2',T)*(Gamma(S_P1_OUT)/Gamma('2'))*Eta('1',M,T) ;
Disj1_Conv_Process1_gt(S_P1_OUT,M,T).. f(S_P1_OUT,T) =G=
                         f('2',T)*(Gamma(S_P1_OUT)/Gamma('2'))*Eta('1',M,T) ;
Disj1_Conv_Process2_lt(S_P2_OUT,M,T).. f(S_P2_OUT,T) =L=
                         f('3',T)*(Gamma(S_P2_OUT)/Gamma('3'))*Eta('2',M,T) ;
Disj1_Conv_Process2_gt(S_P2_OUT,M,T).. f(S_P2_OUT,T) =G=
                         f('3',T)*(Gamma(S_P2_OUT)/Gamma('3'))*Eta('2',M,T) ;
Disj1_Conv_Process3_lt(S_P3_OUT,M,T).. f(S_P3_OUT,T) =L=
                         f('11',T)*(Gamma(S_P3_OUT)/Gamma('11'))*Eta('3',M,T);
Disj1_Conv_Process3_gt(S_P3_OUT,M,T).. f(S_P3_OUT,T) =G=
                         f('11',T)*(Gamma(S_P3_OUT)/Gamma('11'))*Eta('3',M,T);
Disj1_Conv_Process4_lt(S_P4_OUT,M,T).. f(S_P4_OUT,T) =L=
                         f('14',T)*(Gamma(S_P4_OUT)/Gamma('14'))*Eta('4',M,T);
Disj1_Conv_Process4_gt(S_P4_OUT,M,T).. f(S_P4_OUT,T) =G=
                         f('14',T)*(Gamma(S_P4_OUT)/Gamma('14'))*Eta('4',M,T);
Disj1_Conv_Process5_lt(S_P5_OUT,M,T).. f(S_P5_OUT,T) =L=
                         f('17',T)*(Gamma(S_P5_OUT)/Gamma('17'))*Eta('5',M,T);
Disj1_Conv_Process5_gt(S_P5_OUT,M,T).. f(S_P5_OUT,T) =G=
                         f('17',T)*(Gamma(S_P5_OUT)/Gamma('17'))*Eta('5',M,T);
Disj1_Conv_Process6_lt(S_P6_OUT,M,T).. f(S_P6_OUT,T) =L=
                         f('16',T)*(Gamma(S_P6_OUT)/Gamma('16'))*Eta('6',M,T);
Disj1_Conv_Process6_gt(S_P6_OUT,M,T).. f(S_P6_OUT,T) =G=
                         f('16',T)*(Gamma(S_P6_OUT)/Gamma('16'))*Eta('6',M,T);
Disj1_Conv_Process7_lt(S_P7_OUT,M,T).. f(S_P7_OUT,T) =L=
                         f('24',T)*(Gamma(S_P7_OUT)/Gamma('24'))*Eta('7',M,T);
Disj1_Conv_Process7_gt(S_P7_OUT,M,T).. f(S_P7_OUT,T) =G=
                         f('24',T)*(Gamma(S_P7_OUT)/Gamma('24'))*Eta('7',M,T);
Disj1_Conv_Process8_lt(S_P8_OUT,M,T).. f(S_P8_OUT,T) =L=
                         f('30',T)*(Gamma(S_P8_OUT)/Gamma('30'))*Eta('8',M,T);
Disj1_Conv_Process8_gt(S_P8_OUT,M,T).. f(S_P8_OUT,T) =G=
                         f('30',T)*(Gamma(S_P8_OUT)/Gamma('30'))*Eta('8',M,T);
Disj1_Conv_Process9_lt(S_P9_OUT,M,T).. f(S_P9_OUT,T) =L=
                         f('33',T)*(Gamma(S_P9_OUT)/Gamma('33'))*Eta('9',M,T);
Disj1_Conv_Process9_gt(S_P9_OUT,M,T).. f(S_P9_OUT,T) =G=
                         f('33',T)*(Gamma(S_P9_OUT)/Gamma('33'))*Eta('9',M,T);
Disj1_Conv_Process10_lt(S_P10_OUT,M,T).. f(S_P10_OUT,T) =L=
                         f('35',T)*(Gamma(S_P10_OUT)/Gamma('35'))*Eta('10',M,T);
Disj1_Conv_Process10_gt(S_P10_OUT,M,T).. f(S_P10_OUT,T) =G=
                         f('35',T)*(Gamma(S_P10_OUT)/Gamma('35'))*Eta('10',M,T);

Disj1_Cap_Process1(M,T).. SUM(S_P1_IN,mf(S_P1_IN,T)) =L= Cap('1',M,T) ;
Disj1_Cap_Process2(M,T).. SUM(S_P2_IN,mf(S_P2_IN,T)) =L= Cap('2',M,T) ;
Disj1_Cap_Process3(M,T).. SUM(S_P3_IN,mf(S_P3_IN,T)) =L= Cap('3',M,T) ;
Disj1_Cap_Process4(M,T).. SUM(S_P4_IN,mf(S_P4_IN,T)) =L= Cap('4',M,T) ;
Disj1_Cap_Process5(M,T).. SUM(S_P5_IN,mf(S_P5_IN,T)) =L= Cap('5',M,T) ;
Disj1_Cap_Process6(M,T).. SUM(S_P6_IN,mf(S_P6_IN,T)) =L= Cap('6',M,T) ;
Disj1_Cap_Process7(M,T).. SUM(S_P7_IN,mf(S_P7_IN,T)) =L= Cap('7',M,T) ;
Disj1_Cap_Process8(M,T).. SUM(S_P8_IN,mf(S_P8_IN,T)) =L= Cap('8',M,T) ;
Disj1_Cap_Process9(M,T).. SUM(S_P9_IN,mf(S_P9_IN,T)) =L= Cap('9',M,T) ;
Disj1_Cap_Process10(M,T).. SUM(S_P10_IN,mf(S_P10_IN,T)) =L= Cap('10',M,T);

*****************************
*Disjunction 2 constraints
*****************************
Disj2_lt(P,M,T).. convcapcost(P,T) =L= FixedCost_ConvCap(P,M,T);
Disj2_gt(P,M,T).. convcapcost(P,T) =G= FixedCost_ConvCap(P,M,T);

*****************

Equivalence_Energy_Cold(S_COLD,K,T).. q_cold(S_COLD,K,T) =E=
mf(S_COLD,T)*Cp_Cold(S_COLD)*DeltaT_Cold(S_COLD,K,T);
Equivalence_Energy_Hot(S_HOT,K,T).. q_hot(S_HOT,K,T) =E=
mf(S_HOT,T)*Cp_Hot(S_HOT)*DeltaT_Hot(S_HOT,K,T);

****************************
*Disjunction 3 constraints
****************************

Disj3_NoEnergy_Steam_lt(T).. q_steam(T) =L= SUM(K,SUM(S_COLD,q_cold(S_COLD,K,T)));
Disj3_NoEnergy_Steam_gt(T).. q_steam(T) =G= SUM(K,SUM(S_COLD,q_cold(S_COLD,K,T)));

Disj3_NoEnergy_Water_lt(T).. q_water(T) =L= SUM(K,SUM(S_HOT,q_hot(S_HOT,K,T)));
Disj3_NoEnergy_Water_gt(T).. q_water(T) =G= SUM(K,SUM(S_HOT,q_hot(S_HOT,K,T)));

Disj3_Transshipment_lt(K,T).. r(K,T) - r(K-1,T) - q_steam_int(K,T) + q_water_int(K,T)
                  =L= SUM(S_HOT,q_hot(S_HOT,K,T)) - SUM(S_COLD,q_cold(S_COLD,K,T));
Disj3_Transshipment_gt(K,T).. r(K,T) - r(K-1,T) - q_steam_int(K,T) + q_water_int(K,T)
                  =G= SUM(S_HOT,q_hot(S_HOT,K,T)) - SUM(S_COLD,q_cold(S_COLD,K,T));

Disj3_Energy_Steam_lt(T).. q_steam(T) =L= SUM(K,q_steam_int(K,T)) ;
Disj3_Energy_Steam_gt(T).. q_steam(T) =G= SUM(K,q_steam_int(K,T)) ;

Disj3_Energy_Water_lt(T).. q_water(T) =L= r('16',T) + SUM(K,q_water_int(K,T)) ;
Disj3_Energy_Water_gt(T).. q_water(T) =G= r('16',T) + SUM(K,q_water_int(K,T)) ;

****************************
*Disjunction 4 constraints
****************************

Disj4_NoEnergy_lt(T).. energycost(T) =L= FixedCost_Energy('1',T) ;
Disj4_NoEnergy_gt(T).. energycost(T) =G= FixedCost_Energy('1',T) ;

Disj4_Energy_lt(T).. energycost(T) =L= FixedCost_Energy('2',T) ;
Disj4_Energy_gt(T).. energycost(T) =G= FixedCost_Energy('2',T) ;

*****************

LimitingCost(T)..   1E3*(SUM(P,convcapcost(P,T)) + energycost(T) +
SUM(S_RAW,Price_Raw(S_RAW,T)*mf(S_RAW,T)))
                  + Price_Steam*q_steam(T) + Price_Water*q_water(T) =L=
1E3*InvestmentLimit(T);

*Logic Constraints

Sum_y(P,T).. SUM(M,y(P,M,T)) =E= 1;
Sum_w(P,T).. SUM(M,w(P,M,T)) =E= 1;
Sum_x(T).. SUM(MOD,x(MOD,T)) =E= 1;
Sum_v(T).. SUM(MOD,v(MOD,T)) =E= 1;

Logic_y(P,M,T,TAU)$(ORD(M) NE 1 AND ORD(T) LT ORD(TAU))..   y(P,M,T) =L=
y(P,M,TAU);
Logic_w(P,M,T,TAU)$(ORD(M) NE 1 AND ORD(TAU) NE ORD(T))..   w(P,M,T) =L=
w(P,'1',TAU);
Logic_yw_M1(P,M,T)$(ORD(M) EQ 1)..                          y(P,M,T) =L=
w(P,M,T);
Logic_yw_MlessM1(P,M,T)$(ORD(M) NE 1)..                     y(P,M,T) =L=
w(P,M,T) + y(P,M,T-1) + y(P,M,T-2) + y(P,M,T-3);

Logic_x(T,TAU)$(ORD(T) LT ORD(TAU))..                       x('2',T) =L=
x('2',TAU);
Logic_v(T,TAU)$(ORD(T) NE ORD(TAU))..                       v('2',T) =L=
v('1',TAU);
Logic_xv_MOD1(T)..                                          x('1',T) =L=
v('1',T);
Logic_xv_MOD2(T)..                                          x('2',T) =L=
v('2',T) + x('2',T-1) + x('2',T-2) + x('2',T-3);

$ONECHO > "%lm.info%"
disjunction dis1(T),dis2(T),dis3(T),dis4(T),dis5(T),dis6(T),dis7(T),dis8(T),dis9(T),dis10(T),dis20(P,T),dis30(T),dis40(T);

dis1(T) is
   if Y('1','1',T) then
         Disj1_Conv_Process1_lt('4','1',T);
         Disj1_Conv_Process1_gt('4','1',T);
         Disj1_Cap_Process1('1',T);
   elsif Y('1','2',T) then
         Disj1_Conv_Process1_lt('4','2',T);
         Disj1_Conv_Process1_gt('4','2',T);
         Disj1_Cap_Process1('2',T);
   elsif Y('1','3',T) then
         Disj1_Conv_Process1_lt('4','3',T);
         Disj1_Conv_Process1_gt('4','3',T);
         Disj1_Cap_Process1('3',T);
   elsif Y('1','4',T) then
         Disj1_Conv_Process1_lt('4','4',T);
         Disj1_Conv_Process1_gt('4','4',T);
         Disj1_Cap_Process1('4',T);
   endif;

dis2(T) is
   if Y('2','1',T) then
         Disj1_Conv_Process2_lt('8','1',T);
         Disj1_Conv_Process2_gt('8','1',T);
         Disj1_Cap_Process2('1',T);
   elsif Y('2','2',T) then
         Disj1_Conv_Process2_lt('8','2',T);
         Disj1_Conv_Process2_gt('8','2',T);
         Disj1_Cap_Process2('2',T);
   elsif Y('2','3',T) then
          Disj1_Conv_Process2_lt('8','3',T);
         Disj1_Conv_Process2_gt('8','3',T);
         Disj1_Cap_Process2('3',T);
   elsif Y('2','4',T) then
         Disj1_Conv_Process2_lt('8','4',T);
         Disj1_Conv_Process2_gt('8','4',T);
         Disj1_Cap_Process2('4',T);
   endif;

dis3(T) is
   if Y('3','1',T) then
         Disj1_Conv_Process3_lt('12','1',T);
         Disj1_Conv_Process3_gt('12','1',T);
         Disj1_Conv_Process3_lt('13','1',T);
         Disj1_Conv_Process3_gt('13','1',T);
         Disj1_Cap_Process3('1',T);
   elsif Y('3','2',T) then
         Disj1_Conv_Process3_lt('12','2',T);
         Disj1_Conv_Process3_gt('12','2',T);
         Disj1_Conv_Process3_lt('13','2',T);
         Disj1_Conv_Process3_gt('13','2',T);
         Disj1_Cap_Process3('2',T);
   elsif Y('3','3',T) then
         Disj1_Conv_Process3_lt('12','3',T);
         Disj1_Conv_Process3_gt('12','3',T);
         Disj1_Conv_Process3_lt('13','3',T);
         Disj1_Conv_Process3_gt('13','3',T);
         Disj1_Cap_Process3('3',T);
   elsif Y('3','4',T) then
         Disj1_Conv_Process3_lt('12','4',T);
         Disj1_Conv_Process3_gt('12','4',T);
         Disj1_Conv_Process3_lt('13','4',T);
         Disj1_Conv_Process3_gt('13','4',T);
         Disj1_Cap_Process3('4',T);
   endif;

dis4(T) is
   if Y('4','1',T) then
         Disj1_Conv_Process4_lt('15','1',T);
         Disj1_Conv_Process4_gt('15','1',T);
         Disj1_Cap_Process4('1',T);
   elsif Y('4','2',T) then
         Disj1_Conv_Process4_lt('15','2',T);
         Disj1_Conv_Process4_gt('15','2',T);
         Disj1_Cap_Process4('2',T);
   elsif Y('4','3',T) then
         Disj1_Conv_Process4_lt('15','3',T);
         Disj1_Conv_Process4_gt('15','3',T);
         Disj1_Cap_Process4('3',T);
   elsif Y('4','4',T) then
         Disj1_Conv_Process4_lt('15','4',T);
         Disj1_Conv_Process4_gt('15','4',T);
         Disj1_Cap_Process4('4',T);
   endif;

dis5(T) is
   if Y('5','1',T) then
         Disj1_Conv_Process5_lt('18','1',T);
         Disj1_Conv_Process5_gt('18','1',T);
         Disj1_Conv_Process5_lt('19','1',T);
         Disj1_Conv_Process5_gt('19','1',T);
         Disj1_Cap_Process5('1',T);
   elsif Y('5','2',T) then
         Disj1_Conv_Process5_lt('18','2',T);
         Disj1_Conv_Process5_gt('18','2',T);
         Disj1_Conv_Process5_lt('19','2',T);
         Disj1_Conv_Process5_gt('19','2',T);
         Disj1_Cap_Process5('2',T);
   elsif Y('5','3',T) then
         Disj1_Conv_Process5_lt('18','3',T);
         Disj1_Conv_Process5_gt('18','3',T);
         Disj1_Conv_Process5_lt('19','3',T);
         Disj1_Conv_Process5_gt('19','3',T);
         Disj1_Cap_Process5('3',T);
   elsif Y('5','4',T) then
         Disj1_Conv_Process5_lt('18','4',T);
         Disj1_Conv_Process5_gt('18','4',T);
         Disj1_Conv_Process5_lt('19','4',T);
         Disj1_Conv_Process5_gt('19','4',T);
         Disj1_Cap_Process5('4',T);
   endif;

dis6(T) is
   if Y('6','1',T) then
         Disj1_Conv_Process6_lt('25','1',T);
         Disj1_Conv_Process6_gt('25','1',T);
         Disj1_Cap_Process6('1',T);
   elsif Y('6','2',T) then
         Disj1_Conv_Process6_lt('25','2',T);
         Disj1_Conv_Process6_gt('25','2',T);
         Disj1_Cap_Process6('2',T);
   elsif Y('6','3',T) then
         Disj1_Conv_Process6_lt('25','3',T);
         Disj1_Conv_Process6_gt('25','3',T);
         Disj1_Cap_Process6('3',T);
   elsif Y('6','4',T) then
         Disj1_Conv_Process6_lt('25','4',T);
         Disj1_Conv_Process6_gt('25','4',T);
         Disj1_Cap_Process6('4',T);
   endif;

dis7(T) is
   if Y('7','1',T) then
         Disj1_Conv_Process7_lt('27','1',T);
         Disj1_Conv_Process7_gt('27','1',T);
         Disj1_Cap_Process7('1',T);
   elsif Y('7','2',T) then
         Disj1_Conv_Process7_lt('27','2',T);
         Disj1_Conv_Process7_gt('27','2',T);
         Disj1_Cap_Process7('2',T);
   elsif Y('7','3',T) then
         Disj1_Conv_Process7_lt('27','3',T);
         Disj1_Conv_Process7_gt('27','3',T);
         Disj1_Cap_Process7('3',T);
   elsif Y('7','4',T) then
         Disj1_Conv_Process7_lt('27','4',T);
         Disj1_Conv_Process7_gt('27','4',T);
         Disj1_Cap_Process7('4',T);
   endif;

dis8(T) is
   if Y('8','1',T) then
         Disj1_Conv_Process8_lt('31','1',T);
         Disj1_Conv_Process8_gt('31','1',T);
         Disj1_Cap_Process8('1',T);
   elsif Y('8','2',T) then
         Disj1_Conv_Process8_lt('31','2',T);
         Disj1_Conv_Process8_gt('31','2',T);
         Disj1_Cap_Process8('2',T);
   elsif Y('8','3',T) then
         Disj1_Conv_Process8_lt('31','3',T);
         Disj1_Conv_Process8_gt('31','3',T);
         Disj1_Cap_Process8('3',T);
   elsif Y('8','4',T) then
         Disj1_Conv_Process8_lt('31','4',T);
         Disj1_Conv_Process8_gt('31','4',T);
         Disj1_Cap_Process8('4',T);
   endif;

dis9(T) is
   if Y('9','1',T) then
         Disj1_Conv_Process9_lt('34','1',T);
         Disj1_Conv_Process9_gt('34','1',T);
         Disj1_Cap_Process9('1',T);
   elsif Y('9','2',T) then
         Disj1_Conv_Process9_lt('34','2',T);
         Disj1_Conv_Process9_gt('34','2',T);
         Disj1_Cap_Process9('2',T);
   elsif Y('9','3',T) then
         Disj1_Conv_Process9_lt('34','3',T);
         Disj1_Conv_Process9_gt('34','3',T);
         Disj1_Cap_Process9('3',T);
   elsif Y('9','4',T) then
         Disj1_Conv_Process9_lt('34','4',T);
         Disj1_Conv_Process9_gt('34','4',T);
         Disj1_Cap_Process9('4',T);
   endif;

dis10(T) is
   if Y('10','1',T) then
         Disj1_Conv_Process10_lt('36','1',T);
         Disj1_Conv_Process10_gt('36','1',T);
         Disj1_Cap_Process10('1',T);
   elsif Y('10','2',T) then
         Disj1_Conv_Process10_lt('36','2',T);
         Disj1_Conv_Process10_gt('36','2',T);
         Disj1_Cap_Process10('2',T);
   elsif Y('10','3',T) then
         Disj1_Conv_Process10_lt('36','3',T);
         Disj1_Conv_Process10_gt('36','3',T);
         Disj1_Cap_Process10('3',T);
   elsif Y('10','4',T) then
         Disj1_Conv_Process10_lt('36','4',T);
         Disj1_Conv_Process10_gt('36','4',T);
         Disj1_Cap_Process10('4',T);
   endif;

dis20(P,T) is
   if w(P,'1',T) then
         Disj2_lt(P,'1',T);
         Disj2_gt(P,'1',T);
   elsif w(P,'2',T) then
         Disj2_lt(P,'2',T);
         Disj2_gt(P,'2',T);
   elsif w(P,'3',T) then
         Disj2_lt(P,'3',T);
         Disj2_gt(P,'3',T);
   elsif w(P,'4',T) then
         Disj2_lt(P,'4',T);
         Disj2_gt(P,'4',T);
   endif;
 dis30(T) is
   if x('1',T) then
         Disj3_NoEnergy_Steam_lt(T);
         Disj3_NoEnergy_Water_lt(T);
         Disj3_NoEnergy_Steam_gt(T);
         Disj3_NoEnergy_Water_gt(T);
   elsif x('2',T) then
         Disj3_Transshipment_lt(K,T) with ord(K) >= 1;
         Disj3_Transshipment_gt(K,T) with ord(K) >= 1;
         Disj3_Energy_Steam_lt(T);
         Disj3_Energy_Water_lt(T);
         Disj3_Energy_Steam_gt(T);
         Disj3_Energy_Water_gt(T);
   endif;

 dis40(T) is
   if v('1',T) then
         Disj4_NoEnergy_lt(T);
         Disj4_NoEnergy_gt(T);
   elsif v('2',T) then
         Disj4_Energy_lt(T);
         Disj4_Energy_gt(T);
   endif;

$offecho


MODEL RETROFIT_BIGM /ALL/;

OPTION LIMROW = 0;
OPTION LIMCOL = 0;
OPTION OPTCR = 0.0;
OPTION OPTCA = 0.0;
OPTION ITERLIM = 10000000;
OPTION RESLIM = 100000;
OPTION MIP = lmchull;

mf.up(S,T)=100.;
f.up(S,T)=100.;
mf_p1_unreacted.up(T)=100.;
mf_p2_unreacted.up(T)=100.;
mf_p3_unreacted.up(T)=100.;
mf_p4_unreacted.up(T)=100.;
mf_p5_unreacted.up(T)=100.;
mf_p6_unreacted.up(T)=100.;
mf_p7_unreacted.up(T)=100.;
mf_p8_unreacted.up(T)=100.;
mf_p9_unreacted.up(T)=100.;
mf_p10_unreacted.up(T)=100.;

q_cold.up(S_COLD,K,T)=5000.;
q_hot.up(S_HOT,K,T)=5000.;
q_steam.up(T) =5000.;
q_water.up(T)=5000.;
q_steam_int.up(K,T) =5000.;
q_water_int.up(K,T)=5000.;
r.up(K,T)=5000.;

convcapcost.up(P,T) =5000.;
energycost.up(T)  =5000.;
*file fx /"%gams.scrdir%logdisj.scr"/; put_utility fx 'inc' / '%gams.i%'; putclose /;
SOLVE RETROFIT_BIGM USING MIP MAXIMIZING obj;

RETROFIT_BIGM.OPTFILE = 1;