$TITLE Research/Strip Packing Problem (SP-BM)
$OFFSYMXREF
$OFFSYMLIST
$ONINLINE
* This problem corresponds to the optimal solution of
* a problem fitting a set of rectangles of fixed dimensions
* in a strip with fixed width W and variable length L that
* is to be minimized
* Reference: Sawaya, N.S.. and Grossmann I.E.,
*        "A cutting plane method for solving
*        linear generalized disjunctive programming problems".
*        Proceedings of PSE2003, 1032 (2003).
* No rotation of the rectangles is assumed
* In this case we have the dimensions of 12 rectangles.
*
* CORRESPONDS TO  A COMPACT DISJUNCTION FORMULATION using WITH clause

SETS
I /1*12/ /*Number of rectangles*/
D /1*4/ /*Number of disjuncts*/;

ALIAS (I,J);

PARAMETERS
L(I) / 1 = 1, 2 = 2, 3 = 3, 4 = 4, 5 = 5, 6 = 9, 7 = 7,
       8 = 6, 9 = 5, 10 = 12, 11 = 3, 12 = 2/

H(I) / 1 = 10, 2 = 9, 3 = 8, 4 = 4, 5 = 5, 6 = 6, 7 = 7
       8 = 3 , 9 = 2, 10 = 1, 11 = 1, 12 = 3/
;

SCALAR WIDTH /10/;
SCALAR LENGTH /27/;     /*Note: This is obtained from simple heuristic*/


VARIABLES OBJVAL, P;
POSITIVE VARIABLES X(I), Y(I);
BINARY VARIABLES Z(D,I,J);

EQUATIONS    OBJBM, COMMON(I), DISJ1(I,J), DISJ2(I,J), DISJ3(I,J),DISJ4(I,J)
             BIN(I,J);

*Objective Function
OBJBM.. OBJVAL =E= P;

*Common Constraints
COMMON(I).. P =G= X(I)+L(I);

*Disjunctions
DISJ1(I,J)$(ORD(I) LT ORD(J)).. X(I)+L(I)-X(J) =L= 0;
DISJ2(I,J)$(ORD(I) LT ORD(J)).. X(J)+L(J)-X(I) =L= 0;
DISJ3(I,J)$(ORD(I) LT ORD(J)).. -(Y(I)-H(I)-Y(J)) =L= 0;
DISJ4(I,J)$(ORD(I) LT ORD(J)).. -(Y(J)-H(J)-Y(I)) =L= 0;

*Summation of binary variables
BIN(I,J)$(ORD(I) LT ORD(J)).. Z('1',I,J)+Z('2',I,J)+Z('3',I,J)+Z('4',I,J) =E= 1;

*Bounds
X.UP(I) = LENGTH - L(I);
Y.LO(I) = H(I);
Y.UP(I) = WIDTH;

*Model parameters
MODEL RESEARCHSPBM / ALL /;

OPTION LIMROW = 0;
OPTION LIMCOL = 0;
OPTION ITERLIM = 100000000;
OPTION MIP=LMBigM;
OPTION OPTCR=0.0;
OPTION OPTCA=0.0;

$ONECHO > "%lm.info%"
DISJUNCTION DJ(I,J);

DJ(I,J) with (ord(I) < ord(J)) is
IF (Z('1',I,J)) THEN
        DISJ1(I,J);
ELSIF Z('2',I,J) THEN
        DISJ2(I,J);
ELSIF Z('3',I,J) THEN
        DISJ3(I,J);
ELSIF Z('4',I,J) THEN
        DISJ4(I,J);
ENDIF;

$offecho

SOLVE RESEARCHSPBM USING MIP MINIMIZING OBJVAL;

